import assert from 'assert'
import Validators, { url } from '../index'
import getErrorId from './helper'

const ERROR_ID = 'form.errors.url'

function test (value, params) {
  return getErrorId(url(params)(value))
}

describe('Validator: url', function () {
  it('should be invalid when `value` is not a valid url', function () {
    assert.equal(ERROR_ID, test(''))
    assert.equal(ERROR_ID, test('http://'))
    assert.equal(ERROR_ID, test('http://.'))
    assert.equal(ERROR_ID, test('http://..'))
    assert.equal(ERROR_ID, test('http://../'))
    assert.equal(ERROR_ID, test('http://?'))
    assert.equal(ERROR_ID, test('http://??'))
    assert.equal(ERROR_ID, test('http://??/'))
    assert.equal(ERROR_ID, test('http://#'))
    assert.equal(ERROR_ID, test('http://##'))
    assert.equal(ERROR_ID, test('http://##/'))
    assert.equal(ERROR_ID, test('http://foo.bar?q=Spaces should be encoded'))
    assert.equal(ERROR_ID, test('//'))
    assert.equal(ERROR_ID, test('//a'))
    assert.equal(ERROR_ID, test('///a'))
    assert.equal(ERROR_ID, test('///'))
    // assert.equal(ERROR_ID, test('http:///a'))
    // assert.equal(ERROR_ID, test('foo.com'))
    // assert.equal(ERROR_ID, test('rdar://1234'))
    // assert.equal(ERROR_ID, test('h://test'))
    // assert.equal(ERROR_ID, test('http:// shouldfail.com'))
    // assert.equal(ERROR_ID, test(':// should fail'))
    // assert.equal(ERROR_ID, test('http://foo.bar/foo(bar)baz quux'))
    // assert.equal(ERROR_ID, test('ftps://foo.bar/'))
    // assert.equal(ERROR_ID, test('http://-error-.invalid/'))
    // assert.equal(ERROR_ID, test('http://a.b--c.de/'))
    // assert.equal(ERROR_ID, test('http://-a.b.co'))
    // assert.equal(ERROR_ID, test('http://a.b-.co'))
    // assert.equal(ERROR_ID, test('http://1.1.1.1.1'))
    // assert.equal(ERROR_ID, test('http://123.123.123'))
    // assert.equal(ERROR_ID, test('http://3628126748'))
    // assert.equal(ERROR_ID, test('http://.www.foo.bar/'))
    // assert.equal(ERROR_ID, test('http://www.foo.bar./'))
    // assert.equal(ERROR_ID, test('http://.www.foo.bar./'))
  })
  it('should be valid when `value` is a valid url', function () {
    assert.ok(!test('http://foo.com/blah_blah'))
    assert.ok(!test('http://foo.com/blah_blah/'))
    assert.ok(!test('http://foo.com/blah_blah_(wikipedia)'))
    assert.ok(!test('http://foo.com/blah_blah_(wikipedia)_(again)'))
    assert.ok(!test('http://www.example.com/wpstyle/?p=364'))
    assert.ok(!test('https://www.example.com/foo/?bar=baz&inga=42&quux'))
    assert.ok(!test('http://✪df.ws/123'))
    assert.ok(!test('http://userid:password@example.com:8080'))
    assert.ok(!test('http://userid:password@example.com:8080/'))
    assert.ok(!test('http://userid@example.com'))
    assert.ok(!test('http://userid@example.com/'))
    assert.ok(!test('http://userid@example.com:8080'))
    assert.ok(!test('http://userid@example.com:8080/'))
    assert.ok(!test('http://userid:password@example.com'))
    assert.ok(!test('http://userid:password@example.com/'))
    assert.ok(!test('http://142.42.1.1/'))
    assert.ok(!test('http://142.42.1.1:8080/'))
    assert.ok(
      !test(
        'https://userid::password@[2001:db8:85a3::8a2e:370:7334]:483/path/example?foobar=8&abc=2&bcd=5#product=browser+what&stuff'
      )
    )
    assert.ok(!test('http://➡.ws/䨹'))
    assert.ok(!test('http://⌘.ws'))
    assert.ok(!test('http://⌘.ws/'))
    assert.ok(!test('http://foo.com/blah_(wikipedia)#cite-1'))
    assert.ok(!test('http://foo.com/blah_(wikipedia)_blah#cite-1'))
    assert.ok(!test('http://foo.com/unicode_(✪)_in_parens'))
    assert.ok(!test('http://foo.com/(something)?after=parens'))
    assert.ok(!test('http://☺.damowmow.com/'))
    assert.ok(!test('http://code.google.com/events/#&product=browser'))
    assert.ok(!test('http://j.mp'))
    assert.ok(!test('http://foo.bar/?q=Test%20URL-encoded%20stuff'))
    assert.ok(!test('http://مثال.إختبار'))
    assert.ok(!test('http://例子.测试'))
    assert.ok(!test('http://उदाहरण.परीक्षा'))
    assert.ok(!test("http://-.~_!$&'()*+,;=:%40:80%2f::::::@example.com"))
    assert.ok(!test('http://1337.net'))
    assert.ok(!test('http://a.b-c.de'))
    assert.ok(!test('http://223.255.255.254'))
    assert.ok(!test('https://foo_bar.example.com/'))
  })
  it("should be invalid if the protocols don't match", function () {
    assert.equal(ERROR_ID, test('http://google.com', { protocol: 'ftp' }))
    assert.equal(ERROR_ID, test('http://google.com', { protocol: ['ftp', 'https'] }))
    assert.equal(ERROR_ID, test('http://google.com', { protocols: 'https' }))
    assert.equal(ERROR_ID, test('http://google.com', { protocols: ['ftp', 'https'] }))
    assert.equal(ERROR_ID, test('https://www.google.com', { protocol: 'ftp' }))
    assert.equal(ERROR_ID, test('ftp://foo:bar@128.193.1.32:3000/foo?foo=bar', { protocol: 'http' }))
  })
  it('should only be valid with certain protocols', function () {
    assert.ok(!test('ftp://foo.bar/baz', { protocol: 'ftp' }))
    assert.ok(!test('http://google.com', { protocol: 'http' }))
    assert.ok(!test('http://google.com', { protocol: ['http', 'https'] }))
    assert.ok(!test('http://google.com', { protocol: ['ftp', 'http', 'https'] }))
    assert.ok(!test('http://google.com', { protocols: 'http' }))
    assert.ok(!test('http://google.com', { protocols: ['ftp', 'http', 'https'] }))
    assert.ok(!test('https://www.google.com', { protocol: 'https' }))
    assert.ok(!test('ftp://foo:bar@128.193.1.32:3000/foo?foo=bar', { protocol: 'ftp' }))
  })
  it('should be valid with no protocol identifier', function () {
    assert.ok(!test('google.com', { protocolIdentifier: false }))
  })
  it('should use default urlProtocols option', function () {
    let defaultValue = Validators.defaultOptions.urlProtocols

    Validators.defaultOptions.urlProtocols = ['ftp']
    assert.equal(ERROR_ID, test('http://google.com'))
    assert.equal(ERROR_ID, test('https://www.google.com'))
    assert.equal(ERROR_ID, test('http://foo:bar@128.193.1.32:3000/foo?foo=bar'))
    assert.ok(!test('ftp://foo:bar@128.193.1.32:3000/foo?foo=bar'))

    Validators.defaultOptions.urlProtocols = ['http', 'https']
    assert.ok(!test('http://google.com'))
    assert.ok(!test('https://www.google.com'))
    assert.ok(!test('http://foo:bar@128.193.1.32:3000/foo?foo=bar'))
    assert.equal(ERROR_ID, test('ftp://foo:bar@128.193.1.32:3000/foo?foo=bar'))

    Validators.defaultOptions.urlProtocols = defaultValue
  })
  it('should use formatMessage', function () {
    let defaultValue = Validators.formatMessage

    Validators.formatMessage = function (msg) {
      return Object.assign({}, msg, { id: msg.id + '2' })
    }
    assert.equal(ERROR_ID + '2', test('http:/:/'))

    Validators.formatMessage = defaultValue
  })
})
